@@ -26,7 +26,7 @@
 #include "file.h"
 
 #ifndef lint
-FILE_RCSID("@(#)$File: readcdf.c,v 1.65 2017/04/08 20:58:03 christos Exp $")
+FILE_RCSID("@(#)$File: readcdf.c,v 1.71 2018/10/15 16:29:16 christos Exp $")
 #endif
 
 #include <assert.h>
@@ -147,118 +147,118 @@ private int
 cdf_file_property_info(struct magic_set *ms, const cdf_property_info_t *info,
     size_t count, const cdf_directory_t *root_storage)
 {
-        size_t i;
-        cdf_timestamp_t tp;
-        struct timespec ts;
-        char buf[64];
-        const char *str = NULL;
-        const char *s, *e;
-        int len;
+	size_t i;
+	cdf_timestamp_t tp;
+	struct timespec ts;
+	char buf[64];
+	const char *str = NULL;
+	const char *s, *e;
+	int len;
 
-        if (!NOTMIME(ms) && root_storage)
+	if (!NOTMIME(ms) && root_storage)
 		str = cdf_clsid_to_mime(root_storage->d_storage_uuid,
 		    clsid2mime);
 
-        for (i = 0; i < count; i++) {
-                cdf_print_property_name(buf, sizeof(buf), info[i].pi_id);
-                switch (info[i].pi_type) {
-                case CDF_NULL:
-                        break;
-                case CDF_SIGNED16:
-                        if (NOTMIME(ms) && file_printf(ms, ", %s: %hd", buf,
-                            info[i].pi_s16) == -1)
-                                return -1;
-                        break;
-                case CDF_SIGNED32:
-                        if (NOTMIME(ms) && file_printf(ms, ", %s: %d", buf,
-                            info[i].pi_s32) == -1)
-                                return -1;
-                        break;
-                case CDF_UNSIGNED32:
-                        if (NOTMIME(ms) && file_printf(ms, ", %s: %u", buf,
-                            info[i].pi_u32) == -1)
-                                return -1;
-                        break;
-                case CDF_FLOAT:
-                        if (NOTMIME(ms) && file_printf(ms, ", %s: %g", buf,
-                            info[i].pi_f) == -1)
-                                return -1;
-                        break;
-                case CDF_DOUBLE:
-                        if (NOTMIME(ms) && file_printf(ms, ", %s: %g", buf,
-                            info[i].pi_d) == -1)
-                                return -1;
-                        break;
-                case CDF_LENGTH32_STRING:
-                case CDF_LENGTH32_WSTRING:
-                        len = info[i].pi_str.s_len;
-                        if (len > 1) {
-                                char vbuf[1024];
-                                size_t j, k = 1;
+	for (i = 0; i < count; i++) {
+		cdf_print_property_name(buf, sizeof(buf), info[i].pi_id);
+		switch (info[i].pi_type) {
+		case CDF_NULL:
+			break;
+		case CDF_SIGNED16:
+			if (NOTMIME(ms) && file_printf(ms, ", %s: %hd", buf,
+			    info[i].pi_s16) == -1)
+				return -1;
+			break;
+		case CDF_SIGNED32:
+			if (NOTMIME(ms) && file_printf(ms, ", %s: %d", buf,
+			    info[i].pi_s32) == -1)
+				return -1;
+			break;
+		case CDF_UNSIGNED32:
+			if (NOTMIME(ms) && file_printf(ms, ", %s: %u", buf,
+			    info[i].pi_u32) == -1)
+				return -1;
+			break;
+		case CDF_FLOAT:
+			if (NOTMIME(ms) && file_printf(ms, ", %s: %g", buf,
+			    info[i].pi_f) == -1)
+				return -1;
+			break;
+		case CDF_DOUBLE:
+			if (NOTMIME(ms) && file_printf(ms, ", %s: %g", buf,
+			    info[i].pi_d) == -1)
+				return -1;
+			break;
+		case CDF_LENGTH32_STRING:
+		case CDF_LENGTH32_WSTRING:
+			len = info[i].pi_str.s_len;
+			if (len > 1) {
+				char vbuf[1024];
+				size_t j, k = 1;
 
-                                if (info[i].pi_type == CDF_LENGTH32_WSTRING)
-                                    k++;
-                                s = info[i].pi_str.s_buf;
+				if (info[i].pi_type == CDF_LENGTH32_WSTRING)
+				    k++;
+				s = info[i].pi_str.s_buf;
 				e = info[i].pi_str.s_buf + len;
-                                for (j = 0; s < e && j < sizeof(vbuf)
+				for (j = 0; s < e && j < sizeof(vbuf)
 				    && len--; s += k) {
-                                        if (*s == '\0')
-                                                break;
-                                        if (isprint((unsigned char)*s))
-                                                vbuf[j++] = *s;
-                                }
-                                if (j == sizeof(vbuf))
-                                        --j;
-                                vbuf[j] = '\0';
-                                if (NOTMIME(ms)) {
-                                        if (vbuf[0]) {
-                                                if (file_printf(ms, ", %s: %s",
-                                                    buf, vbuf) == -1)
-                                                        return -1;
-                                        }
-                                } else if (str == NULL && info[i].pi_id ==
+					if (*s == '\0')
+						break;
+					if (isprint((unsigned char)*s))
+						vbuf[j++] = *s;
+				}
+				if (j == sizeof(vbuf))
+					--j;
+				vbuf[j] = '\0';
+				if (NOTMIME(ms)) {
+					if (vbuf[0]) {
+						if (file_printf(ms, ", %s: %s",
+						    buf, vbuf) == -1)
+							return -1;
+					}
+				} else if (str == NULL && info[i].pi_id ==
 				    CDF_PROPERTY_NAME_OF_APPLICATION) {
 					str = cdf_app_to_mime(vbuf, app2mime);
 				}
 			}
-                        break;
-                case CDF_FILETIME:
-                        tp = info[i].pi_tp;
-                        if (tp != 0) {
+			break;
+		case CDF_FILETIME:
+			tp = info[i].pi_tp;
+			if (tp != 0) {
 				char tbuf[64];
-                                if (tp < 1000000000000000LL) {
-                                        cdf_print_elapsed_time(tbuf,
-                                            sizeof(tbuf), tp);
-                                        if (NOTMIME(ms) && file_printf(ms,
-                                            ", %s: %s", buf, tbuf) == -1)
-                                                return -1;
-                                } else {
-                                        char *c, *ec;
-                                        cdf_timestamp_to_timespec(&ts, tp);
-                                        c = cdf_ctime(&ts.tv_sec, tbuf);
-                                        if (c != NULL &&
+				if (tp < 1000000000000000LL) {
+					cdf_print_elapsed_time(tbuf,
+					    sizeof(tbuf), tp);
+					if (NOTMIME(ms) && file_printf(ms,
+					    ", %s: %s", buf, tbuf) == -1)
+						return -1;
+				} else {
+					char *c, *ec;
+					cdf_timestamp_to_timespec(&ts, tp);
+					c = cdf_ctime(&ts.tv_sec, tbuf);
+					if (c != NULL &&
 					    (ec = strchr(c, '\n')) != NULL)
 						*ec = '\0';
 
-                                        if (NOTMIME(ms) && file_printf(ms,
-                                            ", %s: %s", buf, c) == -1)
-                                                return -1;
-                                }
-                        }
-                        break;
-                case CDF_CLIPBOARD:
-                        break;
-                default:
-                        return -1;
-                }
-        }
-        if (!NOTMIME(ms)) {
+					if (NOTMIME(ms) && file_printf(ms,
+					    ", %s: %s", buf, c) == -1)
+						return -1;
+				}
+			}
+			break;
+		case CDF_CLIPBOARD:
+			break;
+		default:
+			return -1;
+		}
+	}
+	if (ms->flags & MAGIC_MIME_TYPE) {
 		if (str == NULL)
 			return 0;
-                if (file_printf(ms, "application/%s", str) == -1)
-                        return -1;
-        }
-        return 1;
+		if (file_printf(ms, "application/%s", str) == -1)
+			return -1;
+	}
+	return 1;
 }
 
 private int
@@ -270,7 +270,7 @@ cdf_file_catalog(struct magic_set *ms, const cdf_header_t *h,
 	char buf[256];
 	cdf_catalog_entry_t *ce;
 
-        if (NOTMIME(ms)) {
+	if (NOTMIME(ms)) {
 		if (file_printf(ms, "Microsoft Thumbs.db [") == -1)
 			return -1;
 		if (cdf_unpack_catalog(h, sst, &cat) == -1)
@@ -285,7 +285,7 @@ cdf_file_catalog(struct magic_set *ms, const cdf_header_t *h,
 				return -1;
 			}
 		free(cat);
-	} else {
+	} else if (ms->flags & MAGIC_MIME_TYPE) {
 		if (file_printf(ms, "application/CDFV2") == -1)
 			return -1;
 	}
@@ -296,44 +296,44 @@ private int
 cdf_file_summary_info(struct magic_set *ms, const cdf_header_t *h,
     const cdf_stream_t *sst, const cdf_directory_t *root_storage)
 {
-        cdf_summary_info_header_t si;
-        cdf_property_info_t *info;
-        size_t count;
-        int m;
+	cdf_summary_info_header_t si;
+	cdf_property_info_t *info;
+	size_t count;
+	int m;
 
-        if (cdf_unpack_summary_info(sst, h, &si, &info, &count) == -1)
-                return -1;
+	if (cdf_unpack_summary_info(sst, h, &si, &info, &count) == -1)
+		return -1;
 
-        if (NOTMIME(ms)) {
+	if (NOTMIME(ms)) {
 		const char *str;
 
-                if (file_printf(ms, "Composite Document File V2 Document")
+		if (file_printf(ms, "Composite Document File V2 Document")
 		    == -1)
-                        return -1;
+			return -1;
 
-                if (file_printf(ms, ", %s Endian",
-                    si.si_byte_order == 0xfffe ?  "Little" : "Big") == -1)
-                        return -2;
-                switch (si.si_os) {
-                case 2:
-                        if (file_printf(ms, ", Os: Windows, Version %d.%d",
-                            si.si_os_version & 0xff,
-                            (uint32_t)si.si_os_version >> 8) == -1)
-                                return -2;
-                        break;
-                case 1:
-                        if (file_printf(ms, ", Os: MacOS, Version %d.%d",
-                            (uint32_t)si.si_os_version >> 8,
-                            si.si_os_version & 0xff) == -1)
-                                return -2;
-                        break;
-                default:
-                        if (file_printf(ms, ", Os %d, Version: %d.%d", si.si_os,
-                            si.si_os_version & 0xff,
-                            (uint32_t)si.si_os_version >> 8) == -1)
-                                return -2;
-                        break;
-                }
+		if (file_printf(ms, ", %s Endian",
+		    si.si_byte_order == 0xfffe ?  "Little" : "Big") == -1)
+			return -2;
+		switch (si.si_os) {
+		case 2:
+			if (file_printf(ms, ", Os: Windows, Version %d.%d",
+			    si.si_os_version & 0xff,
+			    (uint32_t)si.si_os_version >> 8) == -1)
+				return -2;
+			break;
+		case 1:
+			if (file_printf(ms, ", Os: MacOS, Version %d.%d",
+			    (uint32_t)si.si_os_version >> 8,
+			    si.si_os_version & 0xff) == -1)
+				return -2;
+			break;
+		default:
+			if (file_printf(ms, ", Os %d, Version: %d.%d", si.si_os,
+			    si.si_os_version & 0xff,
+			    (uint32_t)si.si_os_version >> 8) == -1)
+				return -2;
+			break;
+		}
 		if (root_storage) {
 			str = cdf_clsid_to_mime(root_storage->d_storage_uuid,
 			    clsid2desc);
@@ -344,20 +344,20 @@ cdf_file_summary_info(struct magic_set *ms, const cdf_header_t *h,
 		}
 	}
 
-        m = cdf_file_property_info(ms, info, count, root_storage);
-        free(info);
+	m = cdf_file_property_info(ms, info, count, root_storage);
+	free(info);
 
-        return m == -1 ? -2 : m;
+	return m == -1 ? -2 : m;
 }
 
 #ifdef notdef
 private char *
 format_clsid(char *buf, size_t len, const uint64_t uuid[2]) {
-	snprintf(buf, len, "%.8" PRIx64 "-%.4" PRIx64 "-%.4" PRIx64 "-%.4" 
+	snprintf(buf, len, "%.8" PRIx64 "-%.4" PRIx64 "-%.4" PRIx64 "-%.4"
 	    PRIx64 "-%.12" PRIx64,
 	    (uuid[0] >> 32) & (uint64_t)0x000000000ffffffffULL,
 	    (uuid[0] >> 16) & (uint64_t)0x0000000000000ffffULL,
-	    (uuid[0] >>  0) & (uint64_t)0x0000000000000ffffULL, 
+	    (uuid[0] >>  0) & (uint64_t)0x0000000000000ffffULL,
 	    (uuid[1] >> 48) & (uint64_t)0x0000000000000ffffULL,
 	    (uuid[1] >>  0) & (uint64_t)0x0000fffffffffffffULL);
 	return buf;
@@ -395,10 +395,10 @@ cdf_check_summary_info(struct magic_set *ms, const cdf_info_t *info,
 	size_t j, k;
 
 #ifdef CDF_DEBUG
-        cdf_dump_summary_info(h, scn);
+	cdf_dump_summary_info(h, scn);
 #endif
-        if ((i = cdf_file_summary_info(ms, h, scn, root_storage)) < 0) {
-            *expn = "Can't expand summary_info";
+	if ((i = cdf_file_summary_info(ms, h, scn, root_storage)) < 0) {
+	    *expn = "Can't expand summary_info";
 	    return i;
 	}
 	if (i == 1)
@@ -416,7 +416,7 @@ cdf_check_summary_info(struct magic_set *ms, const cdf_info_t *info,
 				return -1;
 			i = 1;
 		}
-	} else {
+	} else if (ms->flags & MAGIC_MIME_TYPE) {
 		if (str == NULL)
 			str = "vnd.ms-office";
 		if (file_printf(ms, "application/%s", str) == -1)
@@ -436,7 +436,7 @@ private struct sinfo {
 	const char *sections[5];
 	const int  types[5];
 } sectioninfo[] = {
-	{ "Encrypted", "encrypted", 
+	{ "Encrypted", "encrypted",
 		{
 			"EncryptedPackage", "EncryptedSummary",
 			NULL, NULL, NULL,
@@ -448,7 +448,7 @@ private struct sinfo {
 
 		},
 	},
-	{ "QuickBooks", "quickbooks", 
+	{ "QuickBooks", "quickbooks",
 		{
 #if 0
 			"TaxForms", "PDFTaxForms", "modulesInBackup",
@@ -527,7 +527,7 @@ cdf_file_dir_info(struct magic_set *ms, const cdf_dir_t *dir)
 		if (NOTMIME(ms)) {
 			if (file_printf(ms, "CDFV2 %s", si->name) == -1)
 				return -1;
-		} else {
+		} else if (ms->flags & MAGIC_MIME_TYPE) {
 			if (file_printf(ms, "application/%s", si->mime) == -1)
 				return -1;
 		}
@@ -537,58 +537,60 @@ cdf_file_dir_info(struct magic_set *ms, const cdf_dir_t *dir)
 }
 
 protected int
-file_trycdf(struct magic_set *ms, int fd, const unsigned char *buf,
-    size_t nbytes)
+file_trycdf(struct magic_set *ms, const struct buffer *b)
 {
-        cdf_info_t info;
-        cdf_header_t h;
-        cdf_sat_t sat, ssat;
-        cdf_stream_t sst, scn;
-        cdf_dir_t dir;
-        int i;
-        const char *expn = "";
-        const cdf_directory_t *root_storage;
+	int fd = b->fd;
+	const unsigned char *buf = CAST(const unsigned char *, b->fbuf);
+	size_t nbytes = b->flen;
+	cdf_info_t info;
+	cdf_header_t h;
+	cdf_sat_t sat, ssat;
+	cdf_stream_t sst, scn;
+	cdf_dir_t dir;
+	int i;
+	const char *expn = "";
+	const cdf_directory_t *root_storage;
 
-        scn.sst_tab = NULL;
-        info.i_fd = fd;
-        info.i_buf = buf;
-        info.i_len = nbytes;
-        if (ms->flags & (MAGIC_APPLE|MAGIC_EXTENSION))
-                return 0;
-        if (cdf_read_header(&info, &h) == -1)
-                return 0;
+	scn.sst_tab = NULL;
+	info.i_fd = fd;
+	info.i_buf = buf;
+	info.i_len = nbytes;
+	if (ms->flags & (MAGIC_APPLE|MAGIC_EXTENSION))
+		return 0;
+	if (cdf_read_header(&info, &h) == -1)
+		return 0;
 #ifdef CDF_DEBUG
-        cdf_dump_header(&h);
+	cdf_dump_header(&h);
 #endif
 
-        if ((i = cdf_read_sat(&info, &h, &sat)) == -1) {
-                expn = "Can't read SAT";
-                goto out0;
-        }
+	if ((i = cdf_read_sat(&info, &h, &sat)) == -1) {
+		expn = "Can't read SAT";
+		goto out0;
+	}
 #ifdef CDF_DEBUG
-        cdf_dump_sat("SAT", &sat, CDF_SEC_SIZE(&h));
+	cdf_dump_sat("SAT", &sat, CDF_SEC_SIZE(&h));
 #endif
 
-        if ((i = cdf_read_ssat(&info, &h, &sat, &ssat)) == -1) {
-                expn = "Can't read SSAT";
-                goto out1;
-        }
+	if ((i = cdf_read_ssat(&info, &h, &sat, &ssat)) == -1) {
+		expn = "Can't read SSAT";
+		goto out1;
+	}
 #ifdef CDF_DEBUG
-        cdf_dump_sat("SSAT", &ssat, CDF_SHORT_SEC_SIZE(&h));
+	cdf_dump_sat("SSAT", &ssat, CDF_SHORT_SEC_SIZE(&h));
 #endif
 
-        if ((i = cdf_read_dir(&info, &h, &sat, &dir)) == -1) {
-                expn = "Can't read directory";
-                goto out2;
-        }
+	if ((i = cdf_read_dir(&info, &h, &sat, &dir)) == -1) {
+		expn = "Can't read directory";
+		goto out2;
+	}
 
-        if ((i = cdf_read_short_stream(&info, &h, &sat, &dir, &sst,
+	if ((i = cdf_read_short_stream(&info, &h, &sat, &dir, &sst,
 	    &root_storage)) == -1) {
-                expn = "Cannot read short stream";
-                goto out3;
-        }
+		expn = "Cannot read short stream";
+		goto out3;
+	}
 #ifdef CDF_DEBUG
-        cdf_dump_dir(&info, &h, &sat, &ssat, &sst, &dir);
+	cdf_dump_dir(&info, &h, &sat, &ssat, &sst, &dir);
 #endif
 #ifdef notdef
 	if (root_storage) {
@@ -612,7 +614,7 @@ file_trycdf(struct magic_set *ms, int fd, const unsigned char *buf,
 			if (file_printf(ms,
 			    "Hangul (Korean) Word Processor File 5.x") == -1)
 			    return -1;
-		    } else {
+		    } else if (ms->flags & MAGIC_MIME_TYPE) {
 			if (file_printf(ms, "application/x-hwp") == -1)
 			    return -1;
 		    }
@@ -623,10 +625,10 @@ file_trycdf(struct magic_set *ms, int fd, const unsigned char *buf,
 		}
 	}
 
-        if ((i = cdf_read_summary_info(&info, &h, &sat, &ssat, &sst, &dir,
-            &scn)) == -1) {
-                if (errno != ESRCH) {
-                        expn = "Cannot read summary info";
+	if ((i = cdf_read_summary_info(&info, &h, &sat, &ssat, &sst, &dir,
+	    &scn)) == -1) {
+		if (errno != ESRCH) {
+			expn = "Cannot read summary info";
 		}
 	} else {
 		i = cdf_check_summary_info(ms, &info, &h,
@@ -653,25 +655,26 @@ out5:
 	cdf_zero_stream(&scn);
 	cdf_zero_stream(&sst);
 out3:
-        free(dir.dir_tab);
+	free(dir.dir_tab);
 out2:
-        free(ssat.sat_tab);
+	free(ssat.sat_tab);
 out1:
-        free(sat.sat_tab);
+	free(sat.sat_tab);
 out0:
-	if (i == -1) {
-	    if (NOTMIME(ms)) {
+	/* If we handled it already, return */
+	if (i != -1)
+		return i;
+	/* Provide a default handler */
+	if (NOTMIME(ms)) {
 		if (file_printf(ms,
 		    "Composite Document File V2 Document") == -1)
-		    return -1;
-		if (*expn)
-		    if (file_printf(ms, ", %s", expn) == -1)
 			return -1;
-	    } else {
+		if (*expn)
+			if (file_printf(ms, ", %s", expn) == -1)
+				return -1;
+	} else if (ms->flags & MAGIC_MIME_TYPE) {
 		if (file_printf(ms, "application/CDFV2") == -1)
-		    return -1;
-	    }
-	    i = 1;
+			return -1;
 	}
-        return i;
+	return 1;
 }
